/* 
 * Copyright (C) 2000 the xine project
 * 
 * This file is part of xine, a unix video player.
 * 
 * xine is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * xine is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA
 *
 * $Id: metronom.c,v 1.9 2000/12/10 01:15:07 guenter Exp $
 */

#ifdef HAVE_CONFIG_H
#include "config.h"
#endif

#include "xine.h"
#include "metronom.h"
#include <inttypes.h>
#include "monitor.h"
#include <stdio.h>

#define PTS_TAB_NUM 50

typedef struct metronom_priv_s {

  uint32_t        pts_per_frame;
  uint32_t        pts_per_smpls;

  int32_t         video_pts_delta;
  int32_t         audio_pts_delta;

  uint32_t        video_vpts;
  uint32_t        audio_vpts;

  uint32_t        sync_pts;
  uint32_t        sync_vpts;


  int             num_video_vpts_guessed;
  int             num_audio_samples_guessed;

} metronom_priv_t;

metronom_priv_t    gMetronom;

#define AUDIO_SAMPLE_NUM 32768

void metronom_reset () {

  gMetronom.video_vpts             = 0;
  gMetronom.audio_vpts             = 0;

  gMetronom.video_pts_delta        = 0;
  gMetronom.audio_pts_delta        = 0;

  gMetronom.num_video_vpts_guessed    = 1;
  gMetronom.num_audio_samples_guessed = 1;

  gMetronom.sync_pts               = 0;
  gMetronom.sync_vpts              = 0;
}

void metronom_set_video_rate (uint32_t pts_per_frame) {
  gMetronom.pts_per_frame = pts_per_frame;
}

void metronom_set_audio_rate (uint32_t pts_per_smpls) {
  gMetronom.pts_per_smpls = pts_per_smpls;

  xprintf (METRONOM | VERBOSE, "metronom: %d pts per %d samples\n", pts_per_smpls, AUDIO_SAMPLE_NUM);

}

uint32_t metronom_got_video_frame (uint32_t pts) {

  uint32_t vpts;

  if (pts) {
    int32_t diff;

    diff = pts - gMetronom.sync_pts;

    if ((abs(diff)<gMetronom.sync_vpts) && (abs(diff)<450000)) {

      int32_t vpts_diff;
      uint32_t synced_vpts = gMetronom.sync_vpts + diff;

      vpts_diff = synced_vpts - gMetronom.video_vpts;

      xprintf (METRONOM | VERBOSE, "metronom: video calced vpts : %d <=> synced vpts : %d (diff: %d, delta: %d)\n",
	      gMetronom.video_vpts, synced_vpts, vpts_diff, gMetronom.video_pts_delta);

      /* calc delta for wrong framerates */

      gMetronom.video_pts_delta += vpts_diff / (gMetronom.num_video_vpts_guessed);

      if (abs(gMetronom.video_pts_delta) >= 1600) 
	gMetronom.video_pts_delta = 0;

      if (synced_vpts>gMetronom.video_vpts)
	gMetronom.video_vpts = synced_vpts;
      
    } else
      xprintf (METRONOM | VERBOSE, "metronom: too far away from syncpoint - diff: %d\n",diff);

    gMetronom.sync_pts = pts;
    gMetronom.sync_vpts = gMetronom.video_vpts;

    gMetronom.num_video_vpts_guessed = 0;
  }
  
  vpts = gMetronom.video_vpts;
  gMetronom.video_vpts += gMetronom.pts_per_frame + gMetronom.video_pts_delta;
  gMetronom.num_video_vpts_guessed++ ;

  xprintf (METRONOM | VERBOSE, "metronom: video vpts for %10d : %10d\n", pts, vpts);

  return vpts;
}


uint32_t metronom_got_audio_samples (uint32_t pts, uint32_t nsamples) {

  uint32_t vpts;

  if (pts) {
    int32_t diff;

    diff = pts - gMetronom.sync_pts;

    if ((abs(diff)<gMetronom.sync_vpts) && (abs(diff)<450000)) {

      int32_t vpts_diff;
      uint32_t synced_vpts = gMetronom.sync_vpts + diff;

      vpts_diff = synced_vpts - gMetronom.audio_vpts;

      xprintf (METRONOM | VERBOSE, "metronom: audio calced vpts : %d <=> synced vpts : %d (diff: %d, delta: %d)\n",
	      gMetronom.audio_vpts, synced_vpts, vpts_diff, gMetronom.audio_pts_delta);
      /* calc delta for wrong samplerates */

      gMetronom.audio_pts_delta += vpts_diff*AUDIO_SAMPLE_NUM / (gMetronom.num_audio_samples_guessed);
      
      if (abs(gMetronom.audio_pts_delta) >= 10000) 
	gMetronom.audio_pts_delta = 0;
      
      if (synced_vpts>gMetronom.audio_vpts)
	gMetronom.audio_vpts = synced_vpts;
      
    } else
      xprintf (METRONOM | VERBOSE, "metronom: too far away from syncpoint - diff: %d\n",diff);

    gMetronom.sync_pts = pts;
    gMetronom.sync_vpts = gMetronom.audio_vpts;
    gMetronom.num_audio_samples_guessed = 0;
  }
  
  vpts = gMetronom.audio_vpts;
  gMetronom.audio_vpts += nsamples * (gMetronom.audio_pts_delta + gMetronom.pts_per_smpls) / AUDIO_SAMPLE_NUM;
  gMetronom.num_audio_samples_guessed += nsamples;

  xprintf (METRONOM | VERBOSE, "metronom: audio vpts for %10d : %10d\n", pts, vpts);

  return vpts;
}
