/* 
 * Copyright (C) 2000 the xine project
 * 
 * This file is part of xine, a unix video player.
 * 
 * xine is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * xine is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA
 *
 * $Id: video_out_xv.c,v 1.56 2000/12/12 21:01:13 guenter Exp $
 * 
 * video_out_xv.c, X11 video extension interface for xine
 *
 * based on mpeg2dec code from
 * Aaron Holtzman <aholtzma@ess.engr.uvic.ca>
 *
 * Xv image support by Gerd Knorr <kraxel@goldbach.in-berlin.de>
 *
 * 15 & 16 bpp support added by Franck Sicard <Franck.Sicard@solsoft.fr>
 *
 * xine-specific code by Guenter Bartsch <bartscgr@studbox.uni-stuttgart.de>
 * 
 */

#ifdef HAVE_CONFIG_H
#include "config.h"
#endif

#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#include "config.h"
#include "video_out.h"

#ifdef HAVE_XV

#include <X11/Xlib.h>
#include <X11/Xutil.h>
#include <X11/cursorfont.h>
#include <errno.h>

#include <X11/extensions/XShm.h>
#include <X11/extensions/Xv.h>
#include <X11/extensions/Xvlib.h>
#ifdef HAVE_XINERAMA
#include <X11/extensions/Xinerama.h>
#endif
#include <sys/ipc.h>
#include <sys/shm.h>
#include <sys/time.h>

#include "monitor.h"
#include "../libmpeg2/mpeg2.h"
#include "../gui/gui_dnd.h"
#include "../configfile.h"

extern Display *gDisplay;
extern Window   gVideoWin;
extern Pixmap   gXineLogo;
extern int      gXineLogoWidth, gXineLogoHeight;

extern uint32_t xine_debug;


typedef struct xv_image_info_s {
  vo_image_buffer_t  mImageBuffer;
  XvImage           *mImage;
  XShmSegmentInfo    mShminfo;
} xv_image_buffer_t;

typedef struct _xv_globals {
  int             screen;
  Window          window;
  XVisualInfo     vinfo;
  int             depth;
  int             bFullscreen;
  int             bIsFullscreen;
  unsigned int    xv_port;
  unsigned int    xv_format;
  GC              gc;
  int             image_width;
  int             image_height;
  int             image_xoff;
  int             image_yoff;
  int             orig_width; /* image size correct. by ratio */
  int             orig_height;
  int             dest_width;
  int             dest_height;

  uint32_t        ratio;
  int             user_ratio, user_ratio_changed;

  XvImage        *cur_image;

  /*
   * misc (read: fun ;))
   */
  int             bLogoMode;

#define           HIDE_CURSOR 0
#define           SHOW_CURSOR 1
  Cursor          mcursor[2];
  int             current_cursor;

} xv_globals;

xv_globals gXv ;

int nLocks;

#define XLOCK { XLockDisplay (gDisplay); }
#define XUNLOCK { XUnlockDisplay (gDisplay); }

static int get_capabilities_xv () {
  return 0 ;
}

static unsigned char bm_no_data[] = { 0,0,0,0, 0,0,0,0 };

static void create_cursor_xv (Colormap colormap) {
  Pixmap bm_no;
  XColor black, dummy;
    
  /*
   * create empty cursor
   */
  
  bm_no = XCreateBitmapFromData(gDisplay, gXv.window,
				bm_no_data, 8, 8);
  
  XAllocNamedColor(gDisplay, colormap, "black", &black, &dummy);

  gXv.mcursor[0] = XCreatePixmapCursor(gDisplay, bm_no, bm_no,
				       &black, &black,
				       0, 0);
  gXv.mcursor[1]= XCreateFontCursor(gDisplay, XC_left_ptr);

}

static void display_cursor_xv(int state) {

  XDefineCursor(gDisplay, gXv.window, gXv.mcursor[state]);
  gXv.current_cursor = state;
}

#define MWM_HINTS_DECORATIONS   (1L << 1)
#define PROP_MWM_HINTS_ELEMENTS 5
typedef struct _mwmhints {
  uint32_t flags;
  uint32_t functions;
  uint32_t decorations;
  int32_t  input_mode;
  uint32_t status;
} MWMHints;

static void setup_window_xv () {
  char                 *hello = "xine video output";
  Colormap              theCmap = 0;
  XSizeHints            hint;
  XEvent                xev;
  XGCValues             xgcv;
  XColor                background, ignored;
  XSetWindowAttributes  attr;
#ifdef HAVE_XINERAMA
  int                   screens;
  XineramaScreenInfo   *screeninfo;
#endif
  int                   ww=0, wh=0;
  float                 aspect;
  MWMHints              mwmhints;
  Atom                  prop;
  XClassHint           *xv_xclasshint;

  /* printf ("setup_window_xv: lock\n"); */
  XLOCK

  if((xv_xclasshint = XAllocClassHint()) != NULL) {
    xv_xclasshint->res_name = "Xine Xv Video";
    xv_xclasshint->res_class = "Xine";
  }

#ifdef HAVE_XINERAMA
  /* Spark
   * some Xinerama stuff
   * I want to figure out what fullscreen means for this setup
   */
  if((screeninfo = XineramaQueryScreens(gDisplay, &screens))) {
    /* Xinerama Detected */
    xprintf (VERBOSE|VIDEO, 
	     "Display is using Xinerama with %d screens\n", screens);
    xprintf (VERBOSE|VIDEO, 
	     " going to assume we are using the first screen.\n");
    xprintf (VERBOSE|VIDEO, " size of the first screen is %dx%d.\n", 
	     screeninfo[0].width, screeninfo[0].height);
    
  }
  else {
    /* no Xinerama */
    xprintf (VERBOSE|VIDEO, "Display is not using Xinerama.\n");
  } 
#endif

  if (gXv.bFullscreen) {

#ifdef HAVE_XINERAMA
    if (XineramaIsActive(gDisplay)) {
      ww = screeninfo[0].width;
      wh = screeninfo[0].height;
    }
    else {
      ww = DisplayWidth (gDisplay, gXv.screen);
      wh = DisplayHeight (gDisplay, gXv.screen);
    }
#else
    ww = DisplayWidth (gDisplay, gXv.screen);
    wh = DisplayHeight (gDisplay, gXv.screen);
#endif    

    /*
     * zoom to fullscreen
     */

    if (gXv.orig_width < ww) {
      aspect = (float) gXv.orig_width / (float) gXv.orig_height ;

      gXv.dest_width = ww;
      gXv.dest_height = ww / aspect;

      if (gXv.dest_height > wh) {

	gXv.dest_width = wh * aspect;
	gXv.dest_height = wh;
      }

    } else {
      gXv.dest_width   = gXv.orig_width  ; /* FIXME */
      gXv.dest_height  = gXv.orig_height ; /* FIXME */
    }

    gXv.image_xoff = ( ww - gXv.dest_width) / 2;
    gXv.image_yoff = ( wh - gXv.dest_height) / 2;


    if (gXv.window) {

      if (gXv.bIsFullscreen) {
	XUNLOCK
	return;
      }

      XDestroyWindow(gDisplay, gXv.window);
      gXv.window = 0;

    }

    gXv.bIsFullscreen = 1;

    /*
     * open fullscreen window
     */

    if (XAllocNamedColor (gDisplay, DefaultColormap (gDisplay, gXv.screen), 
			  "black",
			  &background, &ignored) == 0) {
      fprintf (stderr, "Cannot allocate color black\n");
      exit(1);
    }

    attr.background_pixel  = background.pixel;
    attr.override_redirect = True;

    gXv.window =
      XCreateWindow (gDisplay, RootWindow (gDisplay, gXv.screen), 0, 0, 
		     ww, wh, 0, gXv.depth,
		     CopyFromParent, DefaultVisual (gDisplay, gXv.screen),
		     CWBackPixel, &attr);
    /* CWBackPixel|CWOverrideRedirect, &attr); */

    if(xv_xclasshint != NULL)
      XSetClassHint(gDisplay, gXv.window, xv_xclasshint);

    gVideoWin = gXv.window;

    /*
     * wm, no borders please
     */
    
    prop = XInternAtom(gDisplay, "_MOTIF_WM_HINTS", True);
    mwmhints.flags = MWM_HINTS_DECORATIONS;
    mwmhints.decorations = 0;
    XChangeProperty(gDisplay, gVideoWin, prop, prop, 32,
		    PropModeReplace, (unsigned char *) &mwmhints,
		    PROP_MWM_HINTS_ELEMENTS);

  } else {

    /*
     * zoom to mpeg1 to double size
     */
    
    if (gXv.orig_width < 600) {
      gXv.dest_width   = gXv.orig_width  *2;
      gXv.dest_height  = gXv.orig_height *2;
    } else {
      gXv.dest_width   = gXv.orig_width  ;
      gXv.dest_height  = gXv.orig_height ;
    }


    if (gXv.window) {

      if (gXv.bIsFullscreen) {
	XDestroyWindow(gDisplay, gXv.window);
	gXv.window = 0;
      } else {
	
	XResizeWindow (gDisplay, gXv.window, gXv.dest_width, 
		       gXv.dest_height);

	XUNLOCK
	return;
	
      }
    }

    gXv.bIsFullscreen = 0;


    gXv.image_xoff = ( ww - gXv.dest_width) / 2;
    gXv.image_yoff = ( wh - gXv.dest_height) / 2;

    hint.x = 0;
    hint.y = 0;
    hint.width  = gXv.dest_width;
    hint.height = gXv.dest_height;
    hint.flags  = PPosition | PSize;

    
    theCmap   = XCreateColormap(gDisplay, RootWindow(gDisplay,gXv.screen), 
			      gXv.vinfo.visual, AllocNone);
  
    attr.background_pixel = 0;
    attr.border_pixel     = 1;
    attr.colormap         = theCmap;
    

    gXv.window = XCreateWindow(gDisplay, RootWindow(gDisplay, gXv.screen),
			       hint.x, hint.y, hint.width, hint.height, 4, 
			       gXv.depth,CopyFromParent,gXv.vinfo.visual,
			       CWBackPixel | CWBorderPixel |CWColormap,&attr);

    if(xv_xclasshint != NULL)
      XSetClassHint(gDisplay, gXv.window, xv_xclasshint);

    gVideoWin = gXv.window;
    
    /* Tell other applications about this window */

    XSetStandardProperties(gDisplay, gXv.window, hello, hello, 
			   None, NULL, 0, &hint);

    gXv.image_xoff = 0;
    gXv.image_yoff = 0;
  }
  
  XSelectInput(gDisplay, gXv.window, StructureNotifyMask | ExposureMask | KeyPressMask | ButtonPressMask | StructureNotifyMask);

  /* Map window. */
  
  XMapRaised(gDisplay, gXv.window);
  
  /* Wait for map. */

  do  {
    XMaskEvent(gDisplay, 
	       StructureNotifyMask, 
	       &xev) ;
  } while (xev.type != MapNotify || xev.xmap.event != gXv.window);

  XFlush(gDisplay);
  XSync(gDisplay, False);
  
  gXv.gc = XCreateGC(gDisplay, gXv.window, 0L, &xgcv);

  if (gXv.bFullscreen) {
    XSetInputFocus (gDisplay, gXv.window, RevertToNone, CurrentTime);
    XMoveWindow (gDisplay, gXv.window, 0, 0);
  }

  /* drag and drop */

  gui_make_window_dnd_aware (gXv.window);

  /*
   * make cursor disappear
   */

  if (gXv.bFullscreen) {
    if (!gXv.mcursor[0]) {
      create_cursor_xv (theCmap);
    }
    //display_cursor_xv(HIDE_CURSOR);
  }

  /* printf ("setup_window_xv: unlock\n"); */
  XUNLOCK 
}

/* setup internal variables and (re-)init window if necessary */
static int set_size_xv (uint32_t width, uint32_t height, uint32_t ratio) {

  double res_h, res_v, display_ratio;

  if ( (gXv.image_width == width) 
       && (gXv.image_height == height)
       && (gXv.ratio == ratio) 
       && (gXv.bFullscreen == gXv.bIsFullscreen)
       && !gXv.user_ratio_changed ) {


    return 0;
  }

  gXv.image_width        = width;
  gXv.image_height       = height;
  gXv.ratio              = ratio;
  gXv.user_ratio_changed = 0;


  /*
   * Mpeg-2:
   */

  res_h = (DisplayWidth (gDisplay, gXv.screen)*1000 / DisplayWidthMM (gDisplay, gXv.screen));

  res_v =  (DisplayHeight (gDisplay, gXv.screen)*1000 / DisplayHeightMM (gDisplay, gXv.screen));

  display_ratio = res_h / res_v;

  xprintf (VERBOSE | VIDEO, "display_ratio : %f\n",display_ratio);

  if (gXv.user_ratio == ASPECT_AUTO) {
    switch (gXv.ratio) {
    case 0: /* forbidden */
      fprintf (stderr, "invalid ratio\n");
      exit (1);
      break;
    case 1: /* square */
      gXv.orig_width = width * display_ratio;
      gXv.orig_height = height;
      break;
    case 2: /* 4:3 */
      gXv.orig_width = width*1.08*display_ratio ;
      gXv.orig_height = height;
      break;
    case 3: /* 16:9 */
      gXv.orig_width = width / 0.7031 *display_ratio;
      gXv.orig_height = height;
      break;
    default:
      gXv.orig_width = width * 1.08 * display_ratio;
      gXv.orig_height = height;
      xprintf (VERBOSE|VIDEO, "invalid ratio\n");
      /*exit (1); */
      break;
    }
  } else if (gXv.user_ratio == ASPECT_ANAMORPHIC) {
    gXv.orig_width = width / 0.7031 ;
    gXv.orig_height = height;
  } else if (gXv.user_ratio == ASPECT_DVB) {
    gXv.orig_width = width * 2 ;
    gXv.orig_height = height;
  } else {
    gXv.orig_width = width;
    gXv.orig_height = height;
  }
    
  xprintf (VERBOSE|VIDEO, "picture size : %d x %d (Ratio: %d)\n",
	   width, height, ratio);

  setup_window_xv () ;

  return 1;
}

static void dispose_image_buffer_xv (vo_image_buffer_t *vo_img) {

  xv_image_buffer_t *img = (xv_image_buffer_t *) vo_img;

  /* printf ("dispose_image_buffer: lock\n"); */
  XLOCK 

  XShmDetach (gDisplay, &img->mShminfo);

  XFree (img->mImage);
  shmdt (img->mShminfo.shmaddr);
  shmctl (img->mShminfo.shmid, IPC_RMID,NULL);

  /* printf ("dispose_image_buffer: unlock\n"); */
  XUNLOCK 
}

static vo_image_buffer_t *alloc_image_buffer_xv () {

  xv_image_buffer_t     *img ;

  /* printf ("alloc_image_buffer: lock\n"); */
  XLOCK

  img = (xv_image_buffer_t *) malloc (sizeof (xv_image_buffer_t));

  if (img==NULL) {
    printf ("out of memory while allocating image buffer\n");
    exit (1);
  }

  img->mImage = XvShmCreateImage(gDisplay, gXv.xv_port, gXv.xv_format, 0,
				 gXv.image_width, gXv.image_height,
				 &img->mShminfo);
  
  if (img->mImage == NULL )  {
    fprintf(stderr, "Shared memory error when allocating image, exit (Ximage error)\n");
    exit (1);
  }

  img->mShminfo.shmid=shmget(IPC_PRIVATE, 
			     img->mImage->data_size, 
			     IPC_CREAT | 0777);

  if (img->mShminfo.shmid < 0 ) {
    perror("shared memory error"); 
    fprintf(stderr, "error when allocating image\n");
    exit (1);
  }

  img->mShminfo.shmaddr  = (char *) shmat(img->mShminfo.shmid, 0, 0);
  
  if (img->mShminfo.shmaddr == ((char *) -1)) {
    fprintf(stderr, "Shared memory error (address error)\n");
    exit (1);
  }

  img->mShminfo.readOnly = False;
  img->mImage->data = img->mShminfo.shmaddr;

  XShmAttach(gDisplay, &img->mShminfo);

  XSync(gDisplay, False);
  shmctl(img->mShminfo.shmid, IPC_RMID, 0);
  
  img->mImageBuffer.mem[0] = img->mImage->data;
  img->mImageBuffer.mem[2] = img->mImage->data + gXv.image_width * gXv.image_height;
  img->mImageBuffer.mem[1] = img->mImage->data + gXv.image_width * gXv.image_height * 5 / 4;
  
  /* printf ("alloc_image_buffer: unlock\n"); */
  XUNLOCK 

  return (vo_image_buffer_t *) img;
}

static void process_macroblock_xv (vo_image_buffer_t *img, 
				   uint8_t *py, uint8_t *pu, uint8_t *pv,
				   int slice_offset,
				   int offset){
}

int is_fullscreen_xv () {
  return gXv.bFullscreen;
}

void set_fullscreen_xv (int bFullscreen) {
  gXv.bFullscreen = bFullscreen;

  /*
  set_size_xv (gXv.image_width, gXv.image_height, gXv.ratio, gXv.rate_code);
  */
  setup_window_xv ();
}


static void display_frame_xv(vo_image_buffer_t *vo_img) {

  xv_image_buffer_t *img = (xv_image_buffer_t *) vo_img;

  /* printf ("display_frame_xv: lock\n"); */
  XLOCK

  XvShmPutImage(gDisplay, gXv.xv_port, gXv.window, gXv.gc, img->mImage,
		0, 0,  gXv.image_width-1, gXv.image_height-1,
		gXv.image_xoff, gXv.image_yoff,  
		gXv.dest_width, gXv.dest_height, False);
  
      
  XFlush(gDisplay); 

  gXv.cur_image = img->mImage;

  /* printf ("display_frame_xv: unlock\n"); */
  XUNLOCK 
}

void draw_logo_xv () {
  XClearWindow (gDisplay, gXv.window); 
    
  XCopyArea (gDisplay, gXineLogo, gXv.window, gXv.gc, 0, 0,
	     gXineLogoWidth, gXineLogoHeight, 
	     (gXv.dest_width - gXineLogoWidth)/2, 
	     (gXv.dest_height - gXineLogoHeight)/2);
	     
  XFlush(gDisplay);
}


void handle_event_xv (XEvent *event) {

  XLOCK
  switch (event->type) {
  case Expose:
    if (event->xexpose.window == gXv.window) {
      if (gXv.bLogoMode) {
	draw_logo_xv ();
      } else {
	XClearWindow (gDisplay, gXv.window); 
	if (gXv.cur_image)
	  XvShmPutImage(gDisplay, gXv.xv_port, gXv.window, gXv.gc, gXv.cur_image,
			0, 0,  gXv.image_width, gXv.image_height-1,
			gXv.image_xoff, gXv.image_yoff,  
			gXv.dest_width, gXv.dest_height, False);
      } 
    }
    break;
  }

  XUNLOCK 
}

void set_logo_mode_xv (int bLogoMode) {

  if (gXv.bLogoMode == bLogoMode)
    return;

  gXv.bLogoMode = bLogoMode;

  
  XLOCK
  if (bLogoMode)
    draw_logo_xv ();
  else { /* FIXME : Bad hack to reinstall Xv overlay */
    XUnmapWindow(gDisplay, gXv.window);
    XMapRaised(gDisplay, gXv.window);

    /* Spark
     * move the window back to 0,0 in case the window manager moved it
     * do this only is the window is fullscreen or
     * we lose the top and left windowborders
     */
    if (gXv.bIsFullscreen) {
      XMoveWindow(gDisplay, gXv.window, 0, 0);
    }

  }
  XUNLOCK 
}

void reset_xv () {
}

void set_aspect_xv (int ratio) {

  ratio %= 4;

  if (ratio != gXv.user_ratio) {
    gXv.user_ratio         = ratio;
    gXv.user_ratio_changed = 1;

    set_size_xv (gXv.image_width, gXv.image_height, gXv.ratio);
  }
}

int get_aspect_xv () {
  return gXv.user_ratio;
}

static vo_driver_t vo_xv = {
  get_capabilities_xv,
  set_size_xv,
  alloc_image_buffer_xv,
  dispose_image_buffer_xv,
  process_macroblock_xv,
  display_frame_xv,
  set_fullscreen_xv,
  is_fullscreen_xv,
  handle_event_xv,
  set_logo_mode_xv,
  reset_xv,
  display_cursor_xv,
  set_aspect_xv,
  get_aspect_xv
};


/* 
 * connect to server, create and map window,
 * allocate colors and (shared) memory
 */

vo_driver_t *init_video_out_xv () {

  XWindowAttributes     attribs;
  unsigned int          adaptors, i, nDefault, formats;
  unsigned int          ver,rel,req,ev,err;
  XvAdaptorInfo        *ai;
  XvImageFormatValues  *fo;
  Atom                  my_atom;
  nLocks = 0;

  /* printf ("init video out: lock\n"); */
  /* XLOCK */

  gXv.image_width=720;
  gXv.image_height=576;

  /*
   * ok, X11, prepare for some video!
   */


  gXv.screen = DefaultScreen(gDisplay);

  XGetWindowAttributes(gDisplay, DefaultRootWindow(gDisplay), &attribs);

  /*
   *
   * depth in X11 terminology land is the number of bits used to
   * actually represent the colour.
   *
   * bpp in X11 land means how many bits in the frame buffer per
   * pixel. 
   *
   * ex. 15 bit color is 15 bit depth and 16 bpp. Also 24 bit
   *     color is 24 bit depth, but can be 24 bpp or 32 bpp.
   */
  
  gXv.depth = attribs.depth;
  
  if (gXv.depth != 15 && gXv.depth != 16 && gXv.depth != 24 && gXv.depth != 32)  {
    /* The root window may be 8bit but there might still be
     * visuals with other bit depths. For example this is the 
     * case on Sun/Solaris machines.
     */
    gXv.depth = 24;
  }

  XMatchVisualInfo(gDisplay, gXv.screen, gXv.depth, TrueColor, &gXv.vinfo);

  /*
   * check for Xvideo support 
   */
  /* XUNLOCK */
  gXv.xv_port = 0;
  if (Success != XvQueryExtension(gDisplay,&ver,&rel,&req,&ev,&err)) {
    xprintf (VERBOSE|VIDEO, "Xvideo extension not present");
    return NULL;
  }
  /* XLOCK */

  if (Success != XvQueryAdaptors(gDisplay,DefaultRootWindow(gDisplay), 
				 &adaptors,&ai))  {
    xprintf(VERBOSE|VIDEO, "Xv: XvQueryAdaptors failed");
    return NULL;
  }

  /* check adaptors */
  for (i = 0; i < adaptors; i++)  {
    if ((ai[i].type & XvInputMask) && (ai[i].type & XvImageMask) && (gXv.xv_port == 0)) 
      gXv.xv_port = ai[i].base_id;
  }

  /* check image formats */
  if (gXv.xv_port != 0)  {
    fo = XvListImageFormats(gDisplay, gXv.xv_port, (int*)&formats);
    
    for(i = 0; i < formats; i++) {
      xprintf(VERBOSE|VIDEO, "Xvideo image format: 0x%x (%4.4s) %s\n", 
	      fo[i].id, (char*)&fo[i].id, 
	      (fo[i].format == XvPacked) ? "packed" : "planar");      
      if (0x32315659 == fo[i].id)  {
	gXv.xv_format = fo[i].id;
	break;
      }
    }
    if (i == formats) /* no matching image format not */
      gXv.xv_port = 0;
  }
  
  if (gXv.xv_port != 0) {
    printf("video_out_xv: using Xvideo port %d for hw scaling\n",
	   gXv.xv_port);
  } else {
    printf("video_out_xv: no video port found\n");
    return NULL;
  }

  /* 
   * Brightness/Satuaration/Contrast settings
   */

  
  my_atom = XInternAtom (gDisplay, "XV_BRIGHTNESS", False);
  XvGetPortAttribute (gDisplay, gXv.xv_port,my_atom, &nDefault);
  XvSetPortAttribute (gDisplay, gXv.xv_port, my_atom, 
		      config_file_lookup_int ("brightness", nDefault));

  my_atom = XInternAtom (gDisplay, "XV_SATURATION", False);
  XvGetPortAttribute (gDisplay, gXv.xv_port,my_atom, &nDefault);
  XvSetPortAttribute (gDisplay, gXv.xv_port, my_atom, 
		      config_file_lookup_int ("saturation", nDefault));

  my_atom = XInternAtom (gDisplay, "XV_CONTRAST", False);
  XvGetPortAttribute (gDisplay, gXv.xv_port,my_atom, &nDefault);
  XvSetPortAttribute (gDisplay, gXv.xv_port, my_atom, 
		      config_file_lookup_int ("contrast", nDefault));
  

  /*
   * init global variables
   */

  gXv.bFullscreen           = 0;
  gXv.bIsFullscreen         = 0;
  gXv.image_width           = 0;
  gXv.image_height          = 0;
  gXv.ratio                 = 0;
  gXv.bLogoMode             = 1;
  gXv.cur_image             = NULL;
  gXv.user_ratio            = ASPECT_AUTO;
  gXv.user_ratio_changed    = 0 ;

  set_size_xv (720,576,1);

  create_cursor_xv(DefaultColormap(gDisplay, 0));
  gXv.current_cursor        = SHOW_CURSOR;

  /* xprintf (VERBOSE|VIDEO, "init video out: unlock\n"); */
  /* XUNLOCK  */

  return &vo_xv;
}

/*  #else  *//* no XV */

/*  vo_functions_t *init_video_out_xv () { */
/*    fprintf (stderr, "Xvideo support not compiled in\n"); */
/*    return NULL; */
/*  } */

#endif /* HAVE_XV */
